'use strict';

var yalwa = yalwa || {};

yalwa.main = (function(pub) {
    
    var lang;
    pub = pub || {};
    pub.maxImageWidth = 1024;
    pub.maxImageHeight = 768;
    pub.init = function() {
        init_language();
        init_page_events();
        if (is_android()) {
            init_webintent();
        }
        init_global_listener();
        pub.init_push_notifications(check_environment());
        update_stats();
        if(check_version()) {
            reset_after_update();
        }
        
        if (window.location.href.search(/\?errorCode=/) !== -1) {
            pub.open_local('error');
        } else {
            pub.tab_open_last(true);
        }
        
        if (is_ios() && window.Keyboard) {
            window.Keyboard.hideFormAccessoryBar(false); // show next/prev/done buttons e.g. in select-box
        }
    };
    
    pub.log_custom_error = function(string) {
        if (typeof window.Muscula !== 'undefined') {
            var error_object = new Error('C:' + pub.get_port() + ' ' + string);
            if (error_object.stack) {
                error_object.stack = error_object.stack.replace(/file:\/\/\//g, ''); // muscula bug: request doesnt show up file file:/// is found within callstack
            }
            window.Muscula.errors.push(error_object);
        }
    };
    
    pub.error_wrapper = function() {
        var args = Array.prototype.slice.call(arguments);
        pub.log_custom_error(JSON.stringify(args));
        window.console.log(args);
        window.console.trace(args);
    };
    
    pub.universal_link = function(event) {
        yalwa.main.log_wrapper('Did launch application from the link: ' + event.detail.url);
        yalwa.main.tab_open(event.detail.url);
    };
    
    pub.init_push_notifications = function(force_update, callback) {
        // default push notifications to be activated
        if (yalwa.main.get('push_enabled') === null) {
            yalwa.main.set('push_enabled', true);
        }
        
        if (yalwa.main.get('push_enabled') && pub.get('userID')) {
            yalwa.main.push = window.PushNotification.init({
                android: {
                    senderID: '373637357546',
                    forceShow: false,
                    clearNotifications: true,
                    clearBadge: true
                },
                ios: {
                    'alert': true,
                    badge: true,
                    sound: true,
                    clearBadge: true
                }
            });
            
            yalwa.main.push.on('registration', function(data) {
                var reg_old = pub.get_pushID();
                var reg_current = data.registrationId;
                if (reg_current) {
                    if (reg_current !== reg_old || force_update) {
                        var promise = yalwa.main.push_registration_serverside_update(reg_current, reg_old);
                        if (callback) {
                            promise.done(callback);
                        }
                        pub.set_pushID(reg_current);
                    }
                }
            });
            
            yalwa.main.push.on('notification', function(data) {
            
                // alert('notification')
                
                // Notification event fired twice in IOS
                // handled in mobile.js
               /* if(pub.get('last_recieved_push_unix') == data.additionalData.unix) {
                    return false;
                }
                pub.set('last_recieved_push_unix', data.additionalData.unix);
                */
                
                // After push notification recieved send tracking to GA via Locanto
                push_try_forwarding(data, is_ios() ? function() {
                    yalwa.main.push.finish(
                      () => {
                        // alert('success');
                      },
                      () => {
                        // alert('error');
                      },
                      data.additionalData.unix
                    );
                } : null);
            });
            
            yalwa.main.push.on('error', function(e) {
                yalwa.main.error_wrapper(e.message);
            });
        }
    };
    
    pub.empty_callback = function() {
    
    };
    
    pub.tab_start = function(url) {//console.log(url)
        if (is_ios()) {
            if(window.tab_hidden_login) {
                if(window.tab_hidden_login.close) {
                    window.tab_hidden_login.close();
                }
                window.tab_hidden_login = null;
            }
            pub.tab = window.cordova.InAppBrowser.open(url, '_blank', 'location=no,useWideViewPort=no,toolbar=no,suppressesIncrementalRendering=no,disallowoverscroll=yes,zoom=no,transitionstyle=crossdissolve,hidespinner=yes');
        } else {
            // custom variable for inAppBrowser in Android isMainView=yes, back button should only hide the app,
            // handle this inappbrowser instance like main-view
            // should not closed
            // only hide it if offline event fired
            
            pub.tab = window.cordova.InAppBrowser.open(url, '_blank', 'location=no,useWideViewPort=no,toolbar=no,suppressesIncrementalRendering=no,disallowoverscroll=yes,zoom=no,isMainView=yes');
        }
        
        tab_add_listener();
        if (is_ios()) {
            pub.tab.addEventListener('loadstop', remove_backbutton_once);
        }
        window.navigator.splashscreen.hide();
    };
    
    pub.logout = function() {
        if (yalwa.main.get_pushID()) {
            yalwa.main.push_registration_serverside_remove();
        }
        yalwa.main.del('userID');
        yalwa.main.del('token');
        yalwa.main.del('nickname');
        yalwa.main.del('primary_email');
        yalwa.main.del('c');
        yalwa.main.set('logged_in', false);
    };
    
    pub.load_country_infos = function(no_cache) {
        //console.log('load_country_infos::')
		var deferred = jQuery.Deferred();
		if(pub.get('trigger_country_list_update')  === true) {
			//console.log('country_list_update_triggered')
			no_cache = true;
		}
		if (no_cache && !pub.is_online()) {
			no_cache = false;
		}
        no_cache = true;
		var unix_now = Math.floor(Date.now() / 1000);
		if (pub.get('country_list', true) && !no_cache && (parseInt(pub.get('country_infos_ts')) > parseInt(unix_now) - (60 * 60 * 24 * 7))) {
		   // console.log('use_cached_country_list')
			deferred.resolve();
		} else {
			if (window.navigator.connection.type === 'none') {
				 pub.open_local('offline');
				 window.navigator.splashscreen.hide();
				// deferred.reject();
			} else {
				//console.log('hier')
				pub.country_list_tdls = ['com', 'ca', 'es', 'co.kr', 'mu', 'co.nz'];
				// pub.country_list_tdls = ['aa', 'bb', 'cc', 'dd', 'ee'];
				country_list_try_tdls().then(data => {
					//console.log('load_country_infos_resolved');
					pub.set('trigger_country_list_update', false);
					deferred.resolve();
				}).fail(error => {
					//console.log('load_country_infos_rejected');
					pub.set('trigger_country_list_update', true);
					deferred.reject();
				})
			}
		}
		
		return deferred.promise();
    };

    //sync ajax loop
    function country_list_try_tdls() {
		var deferred = jQuery.Deferred();
		var i = 0;
		function next() {
			var tdl = yalwa.main.country_list_tdls[i];
			var start_url = 'https://m.locanto.' + tdl + pub.get_port(true) + '/';
			//console.log(start_url)
			if (i < pub.country_list_tdls.length) {
				self.yalwa.main.country_list_ajax(start_url).then(data => {
					//console.log('country_list_try_tdls_resolve');
					deferred.resolve();
				}).fail(error => {
					++i;
					//console.log('country_list_try_tdls_next');
					next();
				});
			} else {
				// all done with loop
				//console.log('country_list_try_tdls_reject');
				deferred.reject();
			}
		}
		next();
		
		return deferred.promise();
    }
    pub.country_list_ajax= function(start_url) {
		var deferred = jQuery.Deferred();
		jQuery.ajax({
			url: start_url + 'api/ajax/general/',
			cache: false,
			data: {
				action: 'country_list'
			},
			dataType: 'json',
			success: function(json){
				if (yalwa.main.is_huawei()) {
					var json_object_keys = Object.keys(json);
					var json_object_values = Object.values(json);
					var filtered_json_array = json_object_values.filter(function(obj, i) {
						if(obj.countryID === '11') { // remove Hong Kong from countries_list
							var country_object_key = json_object_keys[i];
							delete json[country_object_key];
						}
					})
				}
		
				if (json) {
					var unix_now = Math.floor(Date.now() / 1000);
					pub.set('country_infos_ts', unix_now);
					pub.set('country_list', json);
				}
				//console.log('country_list_ajax_resolve');
				deferred.resolve();
			},
			error: function(e) {
				//console.log('country_list_ajax_reject');
				deferred.reject();
			}
		})
		
		return deferred.promise();
    }
    
    pub.is_online = function() {
        return window.navigator.connection.type !== window.Connection.NONE;
    };
    
    pub.tab_open_last = function(dont_exit) {
        // we want to start with the app start that instead of last_url start_url. see Ticket #38251
        // pub.tab_open(pub.get('last_remote_url'), dont_exit ? false : true);
        pub.tab_open(pub.start_url());
    };
    
    pub.login_attempt = function(url) {
        window.login_attempt_url = url;
        reset_tab_hidden();
        window.tab_hidden_login = window.cordova.InAppBrowser.open(get_login_url(url, pub.get('countryID')), '_blank', 'location=no,useWideViewPort=no,toolbar=no,suppressesIncrementalRendering=no,disallowoverscroll=yes,zoom=no,hidden=yes,transitionstyle=crossdissolve');
        window.tab_hidden_login.addEventListener('loadstop', login_load_stop);
        window.tab_hidden_login.addEventListener('loadstart', login_load_start);
        window.tab_hidden_login.addEventListener('exit', login_exit);
        window.tab_hidden_login.addEventListener('loaderror', login_load_error);
    };
    
    pub.tab_open = function(url, potential_exit) {
        if (!pub.is_online()) {
            pub.open_local('offline');
        } else {
            url = url || pub.start_url();
            if (!url) {
                return;
            }
            if (is_android() && potential_exit && url === pub.start_url()) {
                clear_action_polling_timeout();
                window.navigator.app.exitApp();
                return;
            }
            if (pub.tab) {
                if (pub.get('last_remote_url') !== url) {
                    pub.tab.executeScript({
                        code: "if(window.location.href !== '" + url + "') {window.location.href = '" + url + "';}"
                    });
                }
            } else {
                if (pub.get('logged_in')) {
                    pub.login_attempt(url);
                } else {
                    pub.tab_start(url);
                }
            }
        }
    };
    
    pub.open_in_second_webview = function(url, force) {
        if (is_ios()) {
            if(force) {
                url = url + '?embedded=1';
            }
            if (!url_is_external(url) && !force) {
                // open internal page into main view
                return pub.tab_open(url);
            }
            pub.ios_second_view = window.cordova.InAppBrowser2.open(url, '_blank', 'toolbarposition=bottom,closebuttoncolor=#414141,toolbarcolor=#ffffff,location=yes,useWideViewPort=no,suppressesIncrementalRendering=no,disallowoverscroll=yes,zoom=no,transitionstyle=crossdissolve,hidespinner=yes');
      // console.log('pub.ios_second_view: ' + pub.ios_second_view)
   // console.log('yalwa.main.ios_second_view: ' + yalwa.main.ios_second_view)
            pub.ios_second_view.addEventListener("loadstart", function(event) {
// console.log(event)
                // Gives the users such a feeling that the page has responded and is currently loading. but show spinner only briefly
                // console.log('loadstart',event.url)
                pub.spinner.show();
            });
            pub.ios_second_view.addEventListener("loadstop", function(event) {
// console.log(event)
                // console.log('loadstart',event.url)
                pub.spinner.hide();
                // remove close button from CVV page
                if (event.url.search(/CVVHelp.jsp/) !== -1) {
                    pub.ios_second_view.insertCSS({ code: ".Help{display:none;" });
                }
                if(force && !url_is_external(event.url)) {
                    pub.ios_second_view.executeScript({
                        code: "window.setTimeout(function() { \n" +
                        "jQuery('a').each(function() {" +
                        "jQuery(this).attr('target','');" +
                        "var url = jQuery(this).attr('href');" +
                        "if (url && url.search(/\\blocanto\\./) !== -1) {" +
                        "var embed_url = '?embedded=1';" +
                        "if(url.search(/\\?/) !== -1) {" +
                        " embed_url = '&embedded=1';" +
                        "}" +
                        "jQuery(this).attr('href', jQuery(this).attr('href') + embed_url);" +
                        "}" +
                        "})}, 340);"
                
                    }, function() {
                        //console.log('script injected');
                    });
                    return;
                }
                //if opened page is not external turn back to main webView
                if (!url_is_external(event.url)) {
                    pub.ios_second_view.close();
                    // open internal page into main view
                    pub.tab_open(event.url);
                }
            });
            pub.ios_second_view.addEventListener('exit', exit_callback);
            pub.ios_second_view.addEventListener('loaderror', function(event) {
   // console.log(event)
            });

            function exit_callback(event) {
                // console.log(event)
                pub.ios_second_view.removeEventListener('loadstop', function(event){});
                pub.ios_second_view.removeEventListener('loadstart', function(event){});
                pub.ios_second_view.removeEventListener('exit', function(event){});
                pub.ios_second_view.removeEventListener('loaderror', function(event){});
                pub.ios_second_view = null;
                pub.ios_second_view = undefined;
            }
            
        } else {
            if(!webview) {
            return false;
            }
            
            pub.set('sw_URL', url);
            pub.set('sw_force', force ? force : false);
            webview.Show('html/second_webview.html', function(e) {}, function(e) {},false);
            webview.SubscribeCallback(second_webview_exit_success, function(){ console.log('second_webview_exit_error'); })
        }
        
    }
    
    function reset_after_update() {
        pub.del('sw_exit_URL');
        pub.del('last_remote_url');
        pub.del('sw_url');
        pub.del('sw_URL');
        pub.del('country_infos_ts');
        pub.set('trigger_country_list_update', true);
    }
    
    function second_webview_exit_success (params) {
        if(!params[0]) {
            return false;
        }
        if(params[0].back_to_locanto) {
            pub.tab_open(params[0].url);
        }
    }
    
    pub.open_local = function(target) {
        window.navigator.splashscreen.hide();
        if (pub.tab) {
            if(target !== 'main') {
                if (pub.tab.hide) {
                    pub.tab.hide();
                }
            } else {
                if (pub.tab.show) {
                    pub.tab.show();
                }
            }
        }
        var current = current_local();
        //console.log(current, target)
        if (target !== current) {
            jQuery('body').pagecontainer('change', target + '.html');
            onshow(target);
        } else if (target !== 'main') {
            onshow(target);
        }
    };
    
    pub.label = function(label) {
        switch (lang) {
            case 'de':
                return window.getText_de(label);
            case 'fr':
                return window.getText_fr(label);
            case 'es':
                return window.getText_es(label);
            case 'nl':
                return window.getText_nl(label);
            case 'id':
                return window.getText_id(label);
        }
        return window.getText_en(label);
    };
    
    pub.start_url = function() {
        var countryID = pub.get('countryID');
        if (countryID > 0 && countryID < 1000) {
            var start_url = build_start_url(countryID);
            if (start_url) {
                return start_url;
            }
        }
        pub.open_local('startup_country_check');
    };
    
    pub.log_wrapper = function() {
        if (!is_prod()) {
            var args = Array.prototype.slice.call(arguments);
            //console.log(args);
            console.trace('trace');
        }
    };
    
    pub.app_version = function() {
        return '2.7.17';
    };
    
    pub.app_version_int = function() {
        var matches = yalwa.main.app_version().match(/([0-9])\.([0-9])\.([0-9]*$)/);
        if (matches && matches[1] && matches[2] && matches[3]) {
            return parseInt(matches[1] + matches[2] + matches[3], 10);
        }
        return 0;
    }
    
    pub.is_huawei = function() {
        var matches = navigator.userAgent.match(/YalwApp\/([abcd])L\.([0-9])\.([0-9])\.([0-9]|[0-9][0-9])/);
        if (matches && matches[1] === 'd') {
            return true;
        }
        return false;
    }
    
    pub.app_version_label = function() {
        var matches = yalwa.main.app_version().match(/([0-9])\.([0-9])\.([0-9])([0-9])/);
        if (matches && matches[1] && matches[2] && matches[3] && matches[4]) {
            return matches[1] + '.' + matches[2] + '.'  + matches[3];
        }
        return 0;
    }
    
    pub.device_version = function() {
        return window.device.version;
    };
    
    pub.get_server_name = function() {
        return 'prod';
    };
    
    pub.get_port = function(https) {
        if (https) {
            return '';
        } else {
            return '';
        }
    };
    
    pub.save_location = function(countryID, close) {
        
        // AUE Personals exception
        if(countryID == 45) {
            return false;
        }
        pub.set('countryID', countryID);
        if (close && is_ios() && pub.tab) { // get rid of the browse history for the old country (doesnt work for android)
            if(pub.tab.close) {
                pub.tab.close();
            }
            pub.tab = null;
        }
        pub.del('last_remote_url');
    };
    
    pub.set = function(key, value) {
        window.localStorage.setItem(key, typeof(value) === 'object' ? JSON.stringify(value) : value);
    };
    
    pub.get = function(key, is_object) {
        var value = window.localStorage.getItem(key);
        if (value === 'true') {
            return true;
        } else if (value === 'false') {
            return false;
        } else if (value === 'null') {
            return null;
        }
        if (is_object) {
            value = JSON.parse(value);
        }
        return value;
    };
    
    pub.del = function(key) {
        window.localStorage.removeItem(key);
    };
    
    pub.countryID_by_cc = function(cc) {
        var country_list = yalwa.main.get('country_list', true);
        for (var i in country_list) {
            var country_details = country_list[i];
            if (country_details.cc === cc) {
                return country_details.countryID;
            }
        }
    };
    
    pub.get_pushID = function() {
        return is_ios() ? yalwa.main.get('apn_registrationID') : yalwa.main.get('gcm_registrationID');
    };
    
    pub.set_pushID = function(value) {
        return is_ios() ? yalwa.main.set('apn_registrationID', value) : yalwa.main.set('gcm_registrationID', value);
    };
    
    pub.push_registration_serverside_update = function(reg_new, reg_old) {
        var start_url = yalwa.main.start_url();
        if (!start_url || !reg_new) {
            yalwa.main.log_wrapper('invalid push_registration_serverside_update call. reg_new: "' + reg_new + '" reg_old: "' + reg_old + '" start_url: "' + start_url + '"');
            return;
        }
        yalwa.main.log_wrapper('reg_old: ' + reg_old);
        yalwa.main.log_wrapper('reg_new: ' + reg_new);
        return jQuery.ajax({
            url: start_url + 'api/ajax/updateRegistrationID/',
            cache: false,
            data: {
                uid: yalwa.main.get('userID'),
                registration_id: reg_new,
                registration_id_old: reg_old,
                token: yalwa.main.get('token'),
                app_version: yalwa.main.app_version(),
                android_version: yalwa.main.device_version(),
                platform: window.device.platform,
                device_name: window.device.manufacturer + ' / ' + window.device.model
            }
        });
    };
    
    pub.push_registration_serverside_remove = function() {
        var registration_id = yalwa.main.get_pushID();
        var start_url = yalwa.main.start_url();
        if (!start_url || !registration_id) {
            yalwa.main.log_wrapper('invalid push_registration_serverside_remove call. start_url: "' + start_url + '" registration_id: "' + registration_id + '"');
            return;
        }
        jQuery.ajax({
            url: start_url + 'api/ajax/updateRegistrationID/',
            cache: false,
            data: {
                registration_id: registration_id,
                platform: window.device.platform,
                action: 'delete'
            }
        });
        yalwa.main.set_pushID(null);
    };
    
    pub.init_country_select = function(select_empty) {
        var select = jQuery('select.select_cc:visible');
        var country_list = yalwa.main.get('country_list', true);
        var array = [];
        for (var countryID_tmp in country_list) {
            array.push(country_list[countryID_tmp]);
        }
        var options = select_empty ? ['<option value="empty">' + yalwa.main.label('no_geo_found_empty_select') + '</option>'] : [];
        array.sort(function(a, b) {
            if (a.name && b.name) {
                return a.name.localeCompare(b.name);
            } else {
                return 0;
            }
        });
        for (countryID_tmp in array) {
            options.push('<option value="' + array[countryID_tmp].countryID + '">' + array[countryID_tmp].name + '</option>');
        }
        select.html(options.join(''));
        var current_option;
        if (select_empty) {
            current_option = select.find('[value="empty"]');
        } else {
            current_option = select.find('[value="' + yalwa.main.get('countryID') + '"]');
        }
        
        current_option.attr('selected', true);
        jQuery('select.select_cc').selectmenu('refresh');
        if (jQuery('span.select_cc').text().search(',') !== -1) {
            jQuery('span.select_cc').text(current_option.text());
        }
    };
    
    function saveFile(dirEntry, fileData, fileName) {
        //console.log('dirEntry', dirEntry);
        dirEntry.getFile(fileName, {create: true, exclusive: false}, function(fileEntry) {
            //console.log('fileEntry', fileEntry)
            writeFile(fileEntry, fileData);
            
        }, onErrorCreateFile);
    };
    
    function writeFile(fileEntry, dataObj, isAppend) {
        
        // Create a FileWriter object for our FileEntry (log.txt).
        fileEntry.createWriter(function(fileWriter) {
            
            fileWriter.onwriteend = function() {
                /*
                window.plugins.webintent.startActivity({
                    action: window.plugins.webintent.ACTION_VIEW,
                    url: fileEntry.nativeURL.replace('file:', 'content:'),
                    type: 'application/pdf'
                }, function () {}, function (error) {
                  console.log(error);
                });*/
                //console.log("Successful file write...", fileWriter);
                var file = is_android() ? fileEntry.nativeURL.replace('file:', 'content:') : fileEntry.nativeURL; // You can also use a Cordova-style file uri: cdvfile://
                cordova.plugins.fileOpener2.open(
                    file,
                    'application/pdf',
                    {
                        error: function(e) {
                            //console.log('Error status: ' + e.status + ' - Error message: ' + e.message);
                        },
                        success: function() {
                            //console.log('file opened successfully');
                        }
                    }
                );
            };
            
            fileWriter.onerror = function(e) {
                //console.log("Failed file write: " + e.toString());
            };
            
            fileWriter.write(dataObj);
        });
    };
    
    function getSampleFile(dirEntry, url) {
        
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'blob';
        
        xhr.onload = function() {//console.log(this)
            if (this.status == 200) {
                //console.log(this);
                if(this.response.type === 'application/pdf') {
                    var blob = new Blob([this.response], {type: this.response.type});
                } else if(this.response.type === 'application/x-download') {
                    var blob = new Blob([this.response], {type: this.response.type});
                } else {
                    return;
                }
                saveFile(dirEntry, blob, Date.now() + ".pdf");
            }
        };
        xhr.send();
    }
    
    function onErrorLoadFs() {
        console.log("File system fail...");
    }
    
    function onErrorCreateFile(e) {
        console.log("File create fail...: " + e);
    }
    pub.spinner = {}
    pub.spinner.show = function() {
        SpinnerDialog.show();
        window.setTimeout(function(){
            SpinnerDialog.hide();
        },1410)
    }
    pub.spinner.hide = function() {
        SpinnerDialog.hide();
    }
                                                                
    pub.open_partner = function(url) {
        // used no more after 270 update
        // Gives the users such a feeling that the page has responded and is currently loading. but show spinner only briefly
        pub.spinner.show();
        if (is_android()) {
            window.plugins.webintent.startActivity(
                {
                    action: window.plugins.webintent.ACTION_VIEW,
                    url: url
                },
                yalwa.main.empty_callback,
                yalwa.main.empty_callback
            );
        } else if (is_ios()) {
            //window.cordova.InAppBrowser.open(url, '_system');
            // doesn't work properly anymore: opens the url in safari AND inappbrowser apparently (iOS 10), therefore:
            window.OpenUrlExt.open(url);
         
        }
    }
    
    pub.open_external = function(url) {
        //if (is_android()) {
            var storageLocation = '';
            if(is_ios && (url.search(/tel:/) !== -1 || url.search(/sms:/) !== -1 || url.search(/mailto:/) !== -1)) {
                return false;
            } else if(is_ios && url.search(/maps.apple.com\/\?ll/) !== -1) {
                return window.OpenUrlExt.open(url);
            } else if (url.search(/pdf\?s=/) !== -1 || url.search(/invoice\/\?id=/) !== -1) {
                switch (device.platform) {
                    case "Android":
                        storageLocation = cordova.file.externalRootDirectory + "Download";
                        break;
                    case "iOS":
                        storageLocation = cordova.file.documentsDirectory;
                        break;
                }
                window.resolveLocalFileSystemURL(storageLocation, function(downloadDirEntry) {
                    //console.log('downloadDirEntry', downloadDirEntry);
                    getSampleFile(downloadDirEntry, url);
                }, onErrorLoadFs());
                /*                window.requestFileSystem(window.PERSISTENT, 0, function (fs) {
                console.log(fs)
                                    getSampleFile(fs.root, url);
                 
                                }, onErrorLoadFs);*/
                
                return false;
            } else {
                yalwa.main.open_in_second_webview(url, false);
                return false;
            }
            window.plugins.webintent.startActivity(
                {
                    action: window.plugins.webintent.ACTION_VIEW,
                    url: url
                },
                yalwa.main.empty_callback,
                yalwa.main.empty_callback
            );
        /* } else if (is_ios()) {
         
            //window.cordova.InAppBrowser.open(url, '_system');
            // doesn't work properly anymore: opens the url in safari AND inappbrowser apparently (iOS 10), therefore:
            window.OpenUrlExt.open(url);
         
        } */
    };
    
    pub.action = {
        
        openexternal: function(params) {
            pub.open_external(params.url);
        },
        
        openpartner: function(params) {
            pub.open_partner(params.url);
        },
        
        opensecondwebview: function(params) {
            yalwa.main.open_in_second_webview(params.url, true);
        },
        
        country_change: function(params) {
            change_country(parseInt(params.countryID), params.name);
        },
        
        push_change: function(params) {
            if (params.push_enabled) {
                yalwa.main.set('push_enabled', true);
                if (params.reload) {
                    yalwa.main.init_push_notifications(true, reload_current_tab);
                } else {
                    yalwa.main.init_push_notifications(true);
                }
            } else {
                yalwa.main.push_registration_serverside_remove();
                yalwa.main.set('push_enabled', false);
                if (yalwa.main.push) {
                    yalwa.main.push.unregister(function() {
                        if (params.reload) {
                            reload_current_tab();
                        }
                        yalwa.main.log_wrapper('push unregister success');
                    }, function() {
                        yalwa.main.log_wrapper('push unregister failure');
                    });
                }
            }
        },
        
        share: function(params) {
            pub.spinner.show();
            if (params && params.provider) {
                var share_subject = params.labels && params.labels.subject ? params.labels.subject : '';
                var share_text = params.labels && params.labels.text ? params.labels.text : '';
                if(params.is_profile) {
                    share_text = params.labels && params.labels.text ? params.labels.text : '';
                    // Unfortunately, we do not have a hard coded label for it in the app using text label...
                    share_subject = params.labels && params.labels.subject ? params.labels.subject : '';
                }
                if (params.is_plain) {
                    share_text = null;
                }
                social_media_share(
                    params.url || pub.get('last_remote_url'),
                    share_subject,
                    share_text,
                    params.provider
                );
            }
        },
        
        image_request: function(params) {
            if (params && params.source && (params.source === 'camera' || params.source === 'folder')) {
                pub.tab.executeScript({
                    code: basics_call('progress.stop', '')
                });
                if (params.maxWidth) {
                    yalwa.main.maxImageWidth = params.maxWidth;
                }
                if (params.maxHeight) {
                    yalwa.main.maxImageHeight = params.maxHeight;
                }
                get_picture(params.source);
            }
        },
        
        get_stats: function() {
            pub.tab.executeScript({
                code: cordova_call('receive_stats', stats_json())
            });
        },
        
        forbid_app_rating: function(params) {
            pub.set('forbid_app_rating', params.value);
        },
        
        init_settings: function() {
            var transfer = {
                version: yalwa.main.app_version_label() + (yalwa.main.get_server_name() === 'prod' ? '' : yalwa.main.get_server_name()),
                countryID: yalwa.main.get('countryID'),
                push: yalwa.main.get('push_enabled') ? true : false,
                regID: yalwa.main.get_pushID(),
                platform: window.device.platform
            };
            pub.tab.executeScript({
                code: external_call('cordova_settings', ['yalwa', 'cordova_settings'], 'init_with_data', JSON.stringify(transfer))
            });
        },
        
        mc_count: function(params) {
            if (params && params.value !== undefined && params.value !== null) {
                set_badge(params.value);
            }
        },
        
        login_auto: function(params) {
            if (params && params.data) {
                if (pub.get('logged_in')) {
                    if (params.data.countryID && parseInt(params.data.countryID) !== parseInt(yalwa.main.get('countryID'))) {
                        change_country(parseInt(params.data.countryID), null, params.url);
                    } else {
                        pub.login_attempt(params.url);
                    }
                    return;
                }
            }
        },
        
        login_store: function(params) {
            if (params && params.data) {
                if (
                    (
                        !pub.get('logged_in') ||
                        !pub.get('userID') ||
                        !pub.get('token') ||
                        !pub.get('nickname') ||
                        !pub.get('primary_email') ||
                        !pub.get('c')
                    ) &&
                    params.data.userID
                ) {
                    
                    pub.set('logged_in', true);
                    pub.set('userID', params.data.userID);
                    pub.set('token', params.data.token);
                    pub.set('nickname', params.data.nickname);
                    pub.set('primary_email', params.data.primary_email);
                    pub.set('c', params.data.c);
                    pub.init_push_notifications();
                }
            }
        },
        
        country_info: function(params) {
            if (params && parseInt(params.ID) !== parseInt(yalwa.main.get('countryID'))) {
                change_country_light(parseInt(params.ID), params.name);
            }
        },
        
        logout: function() {
            pub.logout();
        },
        
        check_pending_pushes: function() {
            var push_pending = pub.get('push_pending', true) || [];
            if (push_pending.length > 0) {
                push_pending.forEach(function(data) {
                    push_forward(data, is_ios() ? function() {
                        yalwa.main.push.finish(function() {
                        });
                    } : function(){});
                });
                push_pending = [];
                pub.set('push_pending', push_pending);
            }
        },
        
        geolocation: function(params) {
            window.navigator.geolocation.getCurrentPosition(
                function(position) {
                    var coords_json = JSON.stringify({
                        coords: {
                            latitude: position.coords.latitude,
                            longitude: position.coords.longitude,
                            accuracy: position.coords.accuracy
                        },
                        type: params.type
                    });
                    pub.tab.executeScript({
                        code: cordova_call('receive_geolocation_success', coords_json)
                    });
                },
                function(error) {
                    pub.tab.executeScript({
                        code: cordova_call('receive_geolocation_error', error.code)
                    });
                }, {
                    timeout: params.timeout || 10000
                }
            );
        },
        
        clear_storage: function(params) {
            clear_storage(params && params.full);
        },
        
        country_list: function() {
            pub.tab.executeScript({
                code: cordova_call('get_country_list', pub.get('country_list'))
            });
        }
    };
    
    function init_universal_links() {
        document.addEventListener('ul_didLaunchAppFromLink', yalwa.main.universal_link, false);
    }
    
    function init_global_listener() {
        init_universal_links();
        if (is_ios()) { // make it possible to cancel 3rd party login: move away from google/facebook domain
            document.addEventListener('resume', function() {
                /* var last_remote = pub.get('last_remote_url');
                if (pub.tab && pub.tab.executeScript) {
                    pub.tab.executeScript({
                        code: cordova_call('on_resume', '')
                    });
                    if (last_remote) {
                         pub.tab.executeScript({
                            code: "location.href"
                        }, function(url) {
                            if (third_party_login(url) || url_is_external(url)) {
                                pub.tab.executeScript({
                                    code: "window.location.href = '" + last_remote + "';"
                                });
                            }
                        });
                    }
                } */
            }, false);
        }
        window.handleOpenURL = function(url) { // https://github.com/EddyVerbruggen/Custom-URL-scheme
            if (!window.no_resume) {
                window.no_resume = true;
                window.setTimeout(function() {
                    window.no_resume = false;
                }, 5000);
                url = decodeURIComponent(url.replace('locanto://', ''));
                if (url !== pub.get('last_remote_url') && url.search(/file:/) === -1) {
                    yalwa.main.tab_open(url);
                }
            }
        };
    }
    
    function page_count_update() {
        yalwa.main.set('page_count', parseInt(yalwa.main.get('page_count') || 0) + 1);
    }
    
    function update_stats() {
        yalwa.main.set('stats_inits', parseInt(yalwa.main.get('stats_inits') || 0) + 1);
        if (!yalwa.main.get('stats_first_launch')) {
            yalwa.main.set('stats_first_launch', Math.floor(Date.now() / 1000));
        }
        yalwa.main.set('stats_last_launch', Math.floor(Date.now() / 1000));
    }
    
    function stats_json() {
        if (pub.get('forbid_app_rating')) {
            return null;
        }
        return JSON.stringify({
            inits: yalwa.main.get('stats_inits'),
            first_launch: yalwa.main.get('stats_first_launch'),
            last_launch: yalwa.main.get('stats_last_launch'),
            page_count: yalwa.main.get('page_count')
        });
    }
    
    function clear_storage(full) {
        if (full === true) {
            window.localStorage.clear();
            if (!is_prod()) {
                alert('cleared local storage');
            }
        } else {
            pub.del('last_remote_url');
            pub.del('country_list');
            if (!is_prod()) {
                alert('cleared last_remote_url and country_list');
            }
        }
    }
    
    function check_port() {
        var current = pub.get_port();
        if (pub.get('last_port') === null) {
            pub.set('last_port', '');
        }
        if (current !== pub.get('last_port')) {
            clear_storage();
            yalwa.main.logout();
            yalwa.main.log_wrapper('new port!');
        }
        pub.set('last_port', current);
    }
    
    function check_version() {
        var current = pub.app_version();
        var is_new = false;
        if (current !== pub.get('last_version')) {
            is_new = true;
            yalwa.main.log_wrapper('new version!');
        }
        pub.set('last_version', current);
        return is_new;
    }
    
    function check_environment() {
        check_port();
        return check_version();
    }
    
    function is_prod() {
        return pub.get_port() === '';
    }
    
    function init_webintent() {
        window.webintent.getUri(function(uri) {
            

            if (uri) {
                if(uri.search(/www\/html\/utm_source=/)){
                    // App just installed, received play store referrer intent.
                    return;
                } else {
                    pub.tab_open(uri);
                }
            }
        });
        window.webintent.onNewIntent(function(uri) {
            // e.g. when clicking register-confirm in email and open via app
            if (uri && uri.search('locanto:') !== 0 && uri !== pub.get('last_remote_url')) {
                window.setTimeout(function() {
                    pub.tab_open(uri);
                }, 300);
            }
        });
    }
    
    function init_page_events() {
        jQuery('body').pagecontainer({
            show: function(event, ui) {
                onshow(ui.toPage.attr('id'));
            }
        });
        window.setTimeout(function() {
            jQuery('img.logo_main').show();
        }, 10000);
        jQuery('.logo_main').on('click', function() {
            hidden_window_workaround();
        });
    }
    
    function onshow(target) {
        var mapping = {
            startup_country_check: {
                js: 'startup_country_check.js',
                func: 'yalwa.startup_country_check.init'
            },
            offline: {
                js: 'offline.js',
                func: 'yalwa.offline.init'
            },
            startup_country_list: {
                js: 'startup_country_list.js',
                func: 'yalwa.startup_country_list.init'
            },
            error: {
                js: 'error.js',
                func: 'yalwa.error.init'
            },
            network_error: {
                js: 'network_error.js',
                func: 'yalwa.network_error.init'
            }
        };
        switch (target) {
            case 'main':
                window.setTimeout(function() {
                    if (!yalwa.main.tab) {
                        pub.tab_open_last();
                    }
                }, 1000);
                break;
            
            case 'startup_country_check':
            case 'offline':
            case 'startup_country_list':
            case 'error':
            case 'network_error':
                window.setTimeout(function(){
                    head.load('../js/' + mapping[target].js, function() {
                        string_to_function(mapping[target].func).apply(this);
                    });
                },400)
               
                break;
        }
    }
    
    function string_to_function(function_name) {
        var namespaces = function_name.split(".");
        var context = window;
        for (var i = 0; i < namespaces.length; i++) {
            if (typeof(context[namespaces[i]]) === 'undefined') {
                return false;
            }
            context = context[namespaces[i]];
        }
        return context;
    }
    
    function tab_add_listener() {
        pub.tab.addEventListener('loadstart', tab_load);
        pub.tab.addEventListener('loadstop', tab_load);
        pub.tab.addEventListener('exit', tab_load);
        pub.tab.addEventListener('loaderror', tab_load);
    }
    
    function init_language() {
        function langSuccess(language) {
            var lang_tmp = language.value.toLowerCase().substr(0, 2);
            switch (lang_tmp) {
                case 'de':
                case 'nl':
                case 'es':
                case 'fr':
                case 'id':
                    lang = lang_tmp;
                    break;
                default:
                    lang = 'en';
                    break;
            }
        }
        
        function langError() {
            lang = 'en';
            yalwa.main.error_wrapper('langError');
        }
        
        window.navigator.globalization.getPreferredLanguage(
            langSuccess,
            langError
        );
    }
    
    function current_local() {
        return jQuery.mobile.activePage.attr('id');
    }
    
    function parse_url(url) {
        var a = document.createElement('a');
        a.href = url;
        return a;
    }
    
    function tab_load(event) { // console.log(event)
        switch (event.type) {
            case 'loadstart':
                if(event.url.search(/about:blank/) !== -1) {
                    return false;
                }
                if (!url_is_external(event.url)) {
                    // Gives the users such a feeling that the page has responded and is currently loading. but show spinner only briefly
                    pub.spinner.show();
                }
                if (event.url.search(/&openpdf=/) !== -1 || event.url.search(/invoice\/\?id=/) !== -1) {//console.log('pdf file opening');
                    pub.tab_open(event.url, false);
                    return false;
                }
                if (event.url.search(/file:/) !== -1) {
                    return false;
                }
                /*else if (event.url.search(/na.gcsip.com/) !== -1 || event.url.search(/eu.gcsip.nl/) !== -1 || event.url.search(/CVVHelp.jsp/) !== -1 || event.url.search(/sofort.com/) !== -1) {
                                   break;
                               }*/
                clear_action_polling_timeout();
                if (url_is_external(event.url)) {
                    pub.open_external(event.url);
                    //console.log('last_remote_url: ',pub.get('last_remote_url'));
                    pub.spinner.hide();
                    return false;
                } else if (!third_party_login(event.url) && event.url.search(/show=edit-profile/) === -1) {
                    pub.set('last_remote_url', event.url);
                }
                break;
            case 'loadstop':
                pub.spinner.hide();
                if(event.url.search(/about:blank/) !== -1) {
                    return false;
                }
                if(url_is_external(event.url)) {
                    pub.tab.executeScript({
                        code: "window.location.href = '" + pub.get('last_remote_url') + "';"
                    });
                    return false;
                }
                if (event.url.search(/&openpdf=/) !== -1 || event.url.search(/invoice\/\?id=/) !== -1) {
                    //console.log('pdf file opened');
                    clear_action_polling_timeout();
                } else if (event.url.search(/CVVHelp.jsp/) !== -1) {
                    //console.log('CVVHelp_popup_loaded');
                    clear_action_polling_timeout();
                    pub.tab.executeScript({
                        code: "window.setTimeout(function() { \n" +
                            "var close_button = document.querySelector('.Help');\n" +
                            "close_button.addEventListener('click', function(e) {\n" +
                            "\t\t\thistory.back();" +
                            "\t\t})\n" +
                            "}, 340);"
                        
                    }, function() {
                        //console.log('script injected');
                    });
                } else {
                    page_count_update();
                    start_action_polling();
                    pub.first_load_success = true;
                }
                break;
            case 'loaderror':
                
                yalwa.main.error_wrapper(event.code, event.message, event.url);
                if (!pub.is_online()) {
                    pub.open_local('offline');
                    pub.tab.hide();
                } else if (event.code === -2){
                    // see https://developer.android.com/reference/android/webkit/WebViewClient
                    
                    pub.tab_open('about:blank');
                    window.setTimeout(function() {
                        pub.tab.hide();
                        pub.open_local('network_error');
                    },100)
                }
                break;
            case 'exit':
                // wait a bit to see if we visit e.g. settings, if not we want to visit the start page again
                window.setTimeout(function() {
                    if (jQuery.mobile.activePage.attr('id') === 'main') {
                        pub.del('last_remote_url');
                        yalwa.main.tab_open_last();
                    }
                }, 100);
                break;
        }
    }
    
    function social_media_share(url, subject, message, provider) {
        var function_name = null;
        var test_name = null;
        switch (provider) {
            case 'Facebook':
                function_name = 'shareViaFacebook';
                test_name = is_ios() ? 'com.apple.social.facebook' : 'facebook';
                break;
            case 'Twitter':
                function_name = 'shareViaTwitter';
                test_name = is_ios() ? 'com.apple.social.twitter' : 'twitter';
                break;
            case 'WhatsApp':
                function_name = 'shareViaWhatsApp';
                test_name = 'whatsapp';
                break;
            case 'Google Plus':
                if (is_android()) {
                    function_name = 'shareVia';
                    test_name = 'plus';
                }
                break;
        }
        var share_success_with_context = function() {
            share_success(provider);
        };
        if (function_name && test_name) {
            window.plugins.socialsharing.canShareVia(test_name, message, null, url, null, function() {
                if (test_name === 'plus') {
                    window.plugins.socialsharing[function_name](
                        test_name,
                        message,
                        subject,
                        null, // img
                        url,
                        share_success_with_context,
                        share_fail
                    );
                } else {
                    window.plugins.socialsharing[function_name](
                        message,
                        null, // img
                        url,
                        share_success_with_context,
                        share_fail
                    );
                }
            }, function() {
                window.plugins.socialsharing.share(
                    message,
                    subject,
                    null,
                    url,
                    share_success_with_context,
                    share_fail
                );
            });
        } else {
            window.plugins.socialsharing.share(
                message,
                subject,
                null,
                url,
                share_success_with_context,
                share_fail
            );
        }
    }
    
    function is_ios() {
        return window.device.platform === 'iOS';
    }
    
    function is_android() {
        return window.device.platform === 'Android';
    }
    
    function share_finished() {
        pub.tab.executeScript({
            code: cordova_call('share_finished', '')
        });
    }
    
    function share_success() {
        if (is_ios()) {
            // hidden_window_workaround();
        }
        share_finished();
    }
    
    function share_fail() {
        if (is_ios()) {
            // hidden_window_workaround();
        }
        share_finished();
    }
    
    function hidden_window_workaround() {
        /*if (!is_ios()) {
            return false;
        }*/
        window.setTimeout(function() {
            if(yalwa.main.tab && yalwa.main.tab.hide) {
                yalwa.main.tab.hide();
            }
            setTimeout(function() {
                if(yalwa.main.tab && yalwa.main.tab.show) {
                    yalwa.main.tab.show();
                }
            }, 100);
        }, parseInt(window.device.version.charAt(0)) > 8 ? 100 : 1500);
    }
    
    function get_picture_success(imageData) {
        if (yalwa.main.app_version_int() >= 244) {
            resize_image(imageData).then(function(resizedImageData) {
                pub.tab.executeScript({
                    code: cordova_call('cordova_upload_blob_new', '"' + resizedImageData + '"')
                });
            })
        } else {
            pub.tab.executeScript({
                code: basics_call('cordova_upload_blob', '"' + imageData + '"')
            });
        }
    }
    
    function get_picture_fail(message) {
        yalwa.main.log_wrapper('Failed because: ' + message);
    }
    
    function get_picture(type) {
        var options = {
            quality: 100,
            //destinationType: window.Camera.DestinationType.DATA_URL,
            //destinationType: Camera.DestinationType.FILE_URI,
            sourceType: (type === 'camera' ? window.Camera.PictureSourceType.CAMERA : window.Camera.PictureSourceType.PHOTOLIBRARY),
            //allowEdit: true, // #30725
            encodingType: window.Camera.EncodingType.JPEG,
            mediaType: window.Camera.MediaType.PICTURE,
            correctOrientation: true,
            targetWidth: 0,
            targetHeight: 0,
            saveToPhotoAlbum: (type === 'camera' ? true : false)
            //saveToPhotoAlbum: false //to testing don't save images
        }
        if (yalwa.main.app_version_int() >= 244) {
            options.destinationType = Camera.DestinationType.FILE_URI;
            options.allowEdit = true;
        } else {
            options.destinationType = window.Camera.DestinationType.DATA_URL;
            options.allowEdit = false; // #30725
        }
        
        window.navigator.camera.getPicture(get_picture_success, get_picture_fail, options);
        
    }
    
    function resize_image(imageSrc) {
        var deferred = jQuery.Deferred();
        
        var canvas = document.getElementById("canvas");
        var ctx = canvas.getContext("2d");
        var imgTmp = new Image();
        var ratio = 1;
        
        imgTmp.onload = function() {
            var canvasCopy = document.createElement('canvas'),
                copyContext = canvasCopy.getContext('2d');
            //console.log(imgTmp.width, imgTmp.height)
            canvasCopy.width = imgTmp.width
            canvasCopy.height = imgTmp.height
            copyContext.drawImage(imgTmp, 0, 0)
            //console.log(canvasCopy.width, canvasCopy.height)
            ratio = imgTmp.height / imgTmp.width
            //console.log(ratio.toFixed(2))
            //max-height or width 256px
            if (imgTmp.width > imgTmp.height) {
                canvas.width = yalwa.main.maxImageWidth;
                canvas.height = Math.round(yalwa.main.maxImageWidth * ratio.toFixed(2))
            } else {
                canvas.height = yalwa.main.maxImageHeight;
                canvas.width = Math.round(yalwa.main.maxImageHeight / ratio.toFixed(2))
            }
            
            //console.log(canvas.width,canvas.height)
            ctx.drawImage(canvasCopy, 0, 0, canvasCopy.width, canvasCopy.height, 0, 0, canvas.width, canvas.height);
            copyContext.clearRect(0, 0, canvasCopy.width, canvasCopy.height);
            ctx.clearRect(canvasCopy, 0, 0, canvasCopy.width, canvasCopy.height, 0, 0, canvas.width, canvas.height);
            imgTmp.src = '';
            deferred.resolve(canvas.toDataURL("image/jpeg", 1.0));
            
        }
        imgTmp.src = imageSrc;
        
        return deferred.promise();
    }
    
    function get_login_url(continue_url, countryID) {
        var params = [
            'autologin=true',
            'userID=' + pub.get('userID'),
            'primary_email=' + pub.get('primary_email'),
            'c=' + pub.get('c'),
            'continue=' + encodeURIComponent(continue_url),
            'ts=' + Date.now()
        ];
        return 'https://my.locanto.info' + pub.get_port(true) + '/m/login?cid=' + (countryID ? countryID : yalwa.main.get('countryID')) + '&' + params.join('&');
    }
    
    function third_party_login(url) {return false;
        var url_parsed = parse_url(url);
        if (url_parsed.hostname.search(/\b(google|facebook)\./) !== -1) {
            if (url_parsed.href.search(/(login|signin)/i) !== -1) {
                return true;
            }
        }
        return false;
    }
    
    function url_is_external(url) {
        var url_parsed = parse_url(url);
        if (url_parsed.hostname.search(/\blcn\.to$/) !== -1 || url_parsed.hostname.search(/\blocan\.to$/) !== -1 || url_parsed.hostname.search(/\blocanto\./) !== -1 || third_party_login(url)) {
            return false;
        }
        return true;
    }
    
    function is_url_payment_action (url) {
        var url_parsed = parse_url(url);
        if(url_parsed.hostname.search(/\bgcsip\./) !== -1) {
            return true;
        } else {
            return false;
        }
    }
    
    function reload_current_tab() {
        if (pub.tab) {
            pub.tab.executeScript({
                code: "location.reload(1);"
            });
        }
    }
    
    function show_transfer_country_message(country_name) {
        if (!country_name) {
            return;
        }
        window.toastr.success(yalwa.main.label('changing_country_info') + ' ' + country_name, yalwa.main.label('settings_current_location').replace(':', ''), {
            positionClass: 'toast-center-center'
        });
    }
    
    function change_country_light(countryID, country_name) {
        yalwa.main.save_location(countryID, false);
        /*if(pub.tab.hide) {
            pub.tab.hide();
        }*/
        show_transfer_country_message(country_name);
        /*setTimeout(function() {
            if(pub.tab && pub.tab.show) {
                pub.tab.show();
            }
        }, 2000);*/
    }
    
    function change_country(countryID, country_name, url) {
        yalwa.main.save_location(countryID, true);
        if (country_name) {
            show_transfer_country_message(country_name);
        }
        url = url || yalwa.main.start_url();
        yalwa.main.open_local('main');
        if (yalwa.main.get('logged_in')) {
            yalwa.main.login_attempt(url);
        } else {
            //yalwa.main.tab_start(url);
            yalwa.main.tab_open(url);
        }
    }
    
    function clear_action_polling_timeout() {// console.log('clear_action_polling_timeout')
        if (pub.timeout_action_polling) {
            window.clearTimeout(pub.timeout_action_polling);
        }
    }
    
    function schedule_next_poll(timeout) {
    // console.log('schedule_next_poll');
    timeout = 400;
        pub.timeout_action_polling = window.setTimeout(action_poll, timeout);
    }
    
    function start_action_polling() {// console.log('start_action_polling')
        clear_action_polling_timeout();
        window.setTimeout(action_poll, 4000);
    }
    
    function action_poll() {
        var default_timeout = 500;
        if (!window.Keyboard.isVisible && pub.tab) {
            var code = basics_call('cordova_collect', '');
            /*if (yalwa.main.app_version_int() >= 244) {
                var code = cordova_call('cordova_collect', '');
            } else {
                var code = basics_call('cordova_collect', '');
            }*/
            pub.tab.executeScript({
                code: code
            }, function(last_expression) {
                // console.log('last_expression',last_expression);
                // console.log('last_expression[0]',last_expression[0]);
                if (!last_expression[0]) {
                    schedule_next_poll(default_timeout);
                    return;
                }
                var data = last_expression[0];
                // console.log('last_expression[0].items', last_expression[0].items);
                if (!data.items) {
                    schedule_next_poll(data.timeout !== undefined ? data.timeout : default_timeout);
                    return;
                }
                var action_item;
                do {
                    action_item = data.items.shift();
                    // console.log('action_item', action_item)
                    if (action_item) {
                        if (pub.action[action_item.action]) {
                            pub.action[action_item.action].call(null, action_item.params);
                        } else {
                            pub.log_custom_error('undefined action: ' + action_item.action);
                        }
                    }
                } while (action_item);
                schedule_next_poll(default_timeout);
            });
        } else {
            schedule_next_poll(default_timeout);
        }
    }
    
    function build_start_url(countryID) {
        countryID = countryID || pub.get('countryID');
        var country_list = yalwa.main.get('country_list', true);
        if (country_list) {
            return country_list[countryID].base_url;
        } else {
            pub.load_country_infos().then(data => {
                //console.log('build_start_url_done');
            }).fail(error => {
                //console.log('build_start_url_fail');
                yalwa.main.open_local('network_error');
            })
        }
    }
    
    function set_badge(count) {
        if (yalwa.main.push) {
            var count_int = parseInt(count);
            if (isNaN(count_int)) {
                count_int = 0;
            }
            //console.log('badge_count: ', count_int)
            yalwa.main.push.setApplicationIconBadgeNumber(yalwa.main.empty_callback, yalwa.main.empty_callback, count_int);
        }
    }
    
    function basics_call(fname, params) {
        return external_call('basics', ['yalwa', 'basics'], fname, params);
    }
    
    function mobile_call(fname, params) {
        return external_call('mobile', ['yalwa', 'mobile'], fname, params);
    }
    
    function cordova_call(fname, params) {
        return external_call('mobile', ['yalwa', 'mobile', 'cordova'], fname, params);
    }
    
    function namespace_hierarchy_helper(namespace_hierarchy) {
        var namespace_check = [];
        for (var i = 0; i < namespace_hierarchy.length; i++) {
            namespace_check[i] = [];
            for (var j = 0; j <= i; j++) {
                namespace_check[i].push(namespace_hierarchy[j]);
            }
        }
        var namespace_check_all = [];
        for (var i = 0; i < namespace_check.length; i++) {
            namespace_check_all.push(namespace_check[i].join('.'));
        }
        return namespace_check_all;
    }
    
    function external_call(file, namespace_hierarchy, fname, params) {
        var namespace_checks = namespace_hierarchy_helper(namespace_hierarchy);
        var namespace_final = namespace_checks[namespace_checks.length - 1];
        var namespace_checks_string = namespace_checks.join(' && ');
        var code = '' +
            'if (typeof(yalwa) === "object" && ' + namespace_checks_string + ' && ' + namespace_final + '.' + fname + ') {' +
            '    ' + namespace_final + '.' + fname + '(' + params + ');' +
            '} else {' +
            '    if (typeof(head) === "function"){' +
            '        head.ready("' + file + '.js", function() {' +
            '        if (typeof(yalwa) === "object" && ' + namespace_checks_string + ') {' +
            '            ' + namespace_final + '.' + fname + '(' + params + ');' +
            '        }' +
            '        });' +
            '        "head";' +
            '    } else {' +
            '        console.log("' + fname + ' not available");' +
            '        "noyalwa";' +
            '    }' +
            '}';
        
        return code;
    }
    
    function push_forward(data, callback) {
        data.platform = window.device.platform;
        pub.tab.executeScript({
            code: 'window.push_tmp = ' + JSON.stringify(data) + ';' + mobile_call('handle_push', 'window.push_tmp')
        }, callback);
    }
    
    function push_store(data) {
        var push_pending = pub.get('push_pending', true) || [];
        push_pending.push(data);
        pub.set('push_pending', push_pending);
    }
    
    function push_try_forwarding(data, callback) {
        var now = Math.round(new Date().getTime() / 1000);
        var message_unix = parseInt(data.additionalData.unix);
        var queue_unix = parseInt(data.additionalData.ts_queued);
        var delay = (now - message_unix);
        var q_delay = (now - queue_unix);
        var qs_ss_delay = (message_unix - queue_unix);
        data.additionalData.delay = JSON.stringify(delay);
        data.additionalData.qDelay = JSON.stringify(q_delay);
        //console.log('queue_start service_send delay in sec.: ' + qs_ss_delay);
        //console.log('queue_start_delay in sec.: ' + q_delay);
        //console.log('service_answer delay in sec.: ' + delay);
        if(is_ios) {
            if(!pub.tab) {
                pub.waiting_tab = window.setInterval(function() {
                    if(pub.tab && pub.first_load_success) {
                        push_forward(data, callback);
                        window.clearInterval(pub.waiting_tab);
                    } else {
                        // alert('waiting');
                    }
                }, 400)
            } else {
                push_forward(data, callback);
            }
        } else {
                                                                
            if (pub.tab) {
                push_forward(data, callback);
            } else {
                push_store(data);
                if (callback) {
                    callback();
                }
            }
        }
    }
    
    function remove_backbutton_once() {
        pub.tab.executeScript({
            code: cordova_call('remove_backbutton', '')
        });
        pub.tab.removeEventListener('loadstop', remove_backbutton_once);
    }
    
    function login_load_stop() {
        reset_tab_hidden();
        //console.log('tab_hidden_load_stop')
        if (is_android()) {
            window.tab_hidden_login.removeEventListener('loadstop', login_load_stop);
            window.tab_hidden_login.removeEventListener('loadstart', login_load_start);
            window.tab_hidden_login.removeEventListener('exit', login_exit);
            window.tab_hidden_login.removeEventListener('loaderror', login_load_error);
            window.tab_hidden_login = null;
        }
        pub.tab_start(window.login_attempt_url);
    }
    
    function reset_tab_hidden() {
        if (is_ios() && window.tab_hidden_login) {
            window.tab_hidden_login.close();
            window.tab_hidden_login.removeEventListener('loadstop', login_load_stop);
            window.tab_hidden_login.removeEventListener('loadstart', login_load_start);
            window.tab_hidden_login.removeEventListener('exit', login_exit);
            window.tab_hidden_login.removeEventListener('loaderror', login_load_error);
            window.tab_hidden_login = null;
        }
        if (window.tab_hidden_login && !is_prod()) {
            window.tab_hidden_login.executeScript({code: 'document.title="autologin";'});
        }
        
    }
    
    function login_load_error(event) {
        yalwa.main.error_wrapper('login loaderror, code: ' + event.code + ' message: ' + event.message + ' url: ' + event.url);
        window.tab_hidden_login.removeEventListener('loadstop', login_load_stop);
        window.tab_hidden_login.removeEventListener('loadstart', login_load_start);
        window.tab_hidden_login.removeEventListener('exit', login_exit);
        window.tab_hidden_login.removeEventListener('loaderror', login_load_error);
        window.tab_hidden_login = null;
        pub.tab_start(window.login_attempt_url);
    }
    
    function login_exit() {
//        alert('exit ' + Math.floor(Date.now() / 1000) - window.test);
        //console.log('tab_hidden_login_exit')
    }
    
    function login_load_start() {
//        alert(event.url);
    }
    
    return pub;
    
}(yalwa.main));

document.addEventListener('deviceready', function() {
    //console.log('deviceready')
    yalwa.main.load_country_infos().then(data => {
        //console.log('done');
        yalwa.main.init();
    }).fail(error => {
        //console.log('fail');
        yalwa.main.init();
        yalwa.main.open_local('network_error');
    })
}, false);



