/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var app = {
    // Application Constructor
    initialize: function() {
        document.addEventListener('deviceready', this.onDeviceReady.bind(this), false);
    },

    // deviceready Event Handler
    //
    // Bind any cordova events here. Common events are:
    // 'pause', 'resume', etc.
    onDeviceReady: function() {
        this.receivedEvent('deviceready');
		var url = this.get_ls('sw_URL');
        var force = this.get_ls('sw_force');
        if(force) {
        	url = url + '?embedded=1';
		}

        this.ref = window.open(url, '_blank', 'location=no,closebuttoncolor=#414141,toolbarcolor=#ffffff,hidenavigationbuttons=no,hideurlbar=no,location=yes,useWideViewPort=no,suppressesIncrementalRendering=no,disallowoverscroll=yes,zoom=no');
        this.ref.addEventListener("loadstart", function(event) {
			// Gives the users such a feeling that the page has responded and is currently loading. but show spinner only briefly
			// console.log('loadstart',event.url)
			SpinnerDialog.show();
			window.setTimeout(function(){
				SpinnerDialog.hide();
			},2410)
        });
        this.ref.addEventListener("loadstop", function(event) {
			// console.log('loadstart',event.url)
            SpinnerDialog.hide();
			app.set_ls('sw_exit_URL', event.url);
			// remove close button from CVV page
			if (event.url.search(/CVVHelp.jsp/) !== -1) {
				app.ref.insertCSS({ code: ".Help{display:none;" });
			}
			if(force && !app.is_url_external(event.url)) {
				app.ref.executeScript({
					code: "window.setTimeout(function() { \n" +
					"jQuery('a').each(function() {" +
					"jQuery(this).attr('target','');" +
					"var url = jQuery(this).attr('href');" +
					"if (url && url.search(/\\blocanto\\./) !== -1) {" +
					"var embed_url = '?embedded=1';" +
					"if(url.search(/\\?/) !== -1) {" +
					" embed_url = '&embedded=1';" +
					"}" +
					"jQuery(this).attr('href', jQuery(this).attr('href') + embed_url);" +
					"}" +
					"})}, 340);"
					
				}, function() {
					//console.log('script injected');
				});
				return;
			}
			//if opened page is not external turn back to main webView
			if (!app.is_url_external(event.url)) {
				app.ref.close();
				app.ref = undefined;
				webview.Hide(null,null,[{'back_to_locanto':true, 'url': event.url}]);
			}
        });
        this.ref.addEventListener('exit', function(event) {
			app.set_ls('sw_exit_URL', event.url);
			app.set_ls('sw_url', '');
			app.set_ls('sw_force', false);
			app.startWebViewCloseTimeout(); //workaround: when native-dialog is called via external js it causes app crash

        });
		this.ref.addEventListener('loaderror', function(event) {
			//console.log(event)
			app.set_ls('sw_force', false);
			app.set_ls('sw_url', '');
			app.set_ls('sw_exit_URL', event.url);
			//app.startWebViewCloseTimeout(); //workaround: when native-dialog is called via external js it causes app crash
		
		});
    },
	startWebViewCloseTimeout: function() {
    	window.setTimeout(function() {
    		app.ref = undefined;
			//close second webview
			webview.Close();
		},41)
	},
	
    // Update DOM on a Received Event
    receivedEvent: function(id) {

    },
	set_ls: function(key, value) {
		window.localStorage.setItem(key, typeof(value) === 'object' ? JSON.stringify(value) : value);
	},
	get_ls: function(key, is_object) {
		var value = window.localStorage.getItem(key);
		if (value === 'true') {
			return true;
		} else if (value === 'false') {
			return false;
		} else if (value === 'null') {
			return null;
		}
		if (is_object) {
			value = JSON.parse(value);
		}
		return value;
	},
	is_url_external: function(url) {
		var url_parsed = parse_url(url);
		if (url_parsed.hostname.search(/\blcn\.to$/) !== -1 || url_parsed.hostname.search(/\blocan\.to$/) !== -1 || url_parsed.hostname.search(/\blocanto\./) !== -1 || third_party_login(url)) {
			return false;
		}
		return true;
	},
	parse_url: function(url) {
		var a = document.createElement('a');
		a.href = url;
		return a;
	},
	third_party_login: function(url) {
		var url_parsed = app.parse_url(url);
		if (url_parsed.hostname.search(/\b(google|facebook)\./) !== -1) {
			if (url_parsed.href.search(/(login|signin)/i) !== -1) {
				return true;
			}
		}
		return false;
	}
};

app.initialize();
